package org.moock.unity.examples.rooms;

import java.util.ArrayList;
import org.moock.unity.core.Attribute;
import org.moock.unity.core.ClientServices;
import org.moock.unity.core.Message;
import org.moock.unity.core.RoomDefinition;
import org.moock.unity.core.RoomListener;
import org.moock.unity.core.RoomServices;
import org.moock.unity.core.Services;
import org.moock.unity.core.exception.NamespaceNotFoundException;
import org.moock.unity.core.exception.RoomAlreadyExistsException;
import org.moock.unity.core.exception.RoomNotFoundException;
import org.moock.unity.core.exception.SpawnRoomException;
import org.moock.unity.core.upc.UPCBuilder;
import org.moock.unity.core.upc.UPCMessage;
import org.moock.unity.core.upc.UPCRoom;
import org.apache.log4j.Logger;

/**
 * This RoomListener can create Custom Rooms.  It extends UPCRoom so contains
 * all of UPCRoom functionality as well.
 *
 * Place the compiled class in .\rooms\org\moock\unity\examples\rooms
 * For example, on Windows XP, compile this file with:
 * javac -classpath lib\unity_core.jar;lib\log4j.jar CustomRoomCreator.java
 * 
 * Usage: Assuming you have the following deployed in your uconfig.xml
 * 
 *  <TYPES>     
 *      <ROOM>
 *          <ID>customRoomCreator</ID>
 *          <CLASS>org.moock.unity.examples.rooms.CustomRoomCreator</CLASS>
 *      </ROOM>
 *      <ROOM>
 *          <ID>helloWorld</ID>
 *          <CLASS>org.moock.unity.examples.rooms.HelloWorld</CLASS>
 *      </ROOM>
 *  </TYPES>  
 *  <INSTANCES>  
 *      <ROOM>
 *          <ID>customRoomCreator</ID>
 *          <TYPE_ID>customRoomCreator</TYPE_ID>
 *          <ACCEPT_OUTSIDE_MESSAGES>true</ACCEPT_OUTSIDE_MESSAGES>
 *      </ROOM>
 *  </INSTANCES> 
 * 
 * From any UClient subclass you could create a HelloWorld type room with:
 * 
 * sendUPC("customRoomCreator", "aHelloWorldRoom", null, "true", null, "helloWorld");
 * 
 * The arguments sent with sendUPC are (not including the first argument which is the name of the Room
 * to send the UPC to...in the above example customRoomCreator):
 * 
 * arg1: the RoomID of the created Room 
 * arg2: the namespace in which you want the Room created (if null uses udefault)
 * arg3: if "false" the Room will not be killed when empty otherwise it will
 * arg4: the maximum number of clients that can be in the Room
 * arg5: the type of Room to be created as defined in uconfig.xml 
 */
public class CustomRoomCreator extends UPCRoom implements RoomListener {   
    private static Logger s_log = Logger.getLogger("Unity");

    public void handleMessage(UPCMessage upcMessage, Message message) {
        if (upcMessage.getMethod().equals("createCustomRoom")) {
            String roomID = upcMessage.getArgText(0);
            String namespace = upcMessage.getArgText(1);
            String dieOnEmpty = upcMessage.getArgText(2);
            String roomType = upcMessage.getArgText(4);       
            String status = null;
            
            int maxClients = -1;
            try {
                maxClients = upcMessage.getArgText(3) != null ? Integer.parseInt(upcMessage.getArgText(3)) : -1;
            } catch (NumberFormatException e) {
                // --- leave as -1
            }

            // --- try and create the room
            RoomDefinition defs = new RoomDefinition(roomType, roomID);           
            if (namespace != null) {
                defs.setNamespace(namespace);
            }
            defs.setDieOnEmpty(dieOnEmpty != null && dieOnEmpty.equals("false") ? false : true);
            defs.setMaxClients(maxClients);      
       
            // --- check if namespace is valid
            try {
                Services.getNamespaceServices(defs.getNamespace());
                try {
                    String fqRoomID = Services.getRoomManagementServices().spawnRoom(defs);
                    try {
                        RoomServices roomServices = Services.getRoomServices(fqRoomID);               
                        roomID = roomServices.getRoomID();            
                        
                        // --- set the owner of the Room
                        UPCRoom upcRoom = (UPCRoom)roomServices.getRoomListener();
                        upcRoom.setOwner(message.getClientServices().getClientID());
                    } catch (RoomNotFoundException e) {
                        s_log.error("Room " + fqRoomID + " was created but could not get it's ID.");
                    }
                    
                    status = "ROOM_CREATED";
                } catch (RoomAlreadyExistsException e) {
                    status = "ROOM_EXISTS";
                } catch (SpawnRoomException e) {
                    s_log.error("Room creation failed: " + e.getMessage());
                    status = "ROOM_CREATION_FAILED";
                }
            } catch (NamespaceNotFoundException e) {
                status = "NAMESPACE_NOT_FOUND";
            }
            
            // --- create a new UPCMessage to send to client
            UPCMessage upcOUT = new UPCMessage("upcOnCreateRoom");
            upcOUT.addArg(status);
            upcOUT.addArg(roomID, true);
            upcOUT.addArg(upcMessage.getArgSource(1));
            message.getClientServices().broadcastClient(UPCBuilder.build(upcOUT));
        }         
    }    
}
